﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using System.ComponentModel;
using System.Windows.Threading;

namespace UndoOperations
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public ObservableCollection<Employee> list;

        public MainWindow()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(MainWindow_Loaded);
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            list = new ObservableCollection<Employee>()
            {
                new Employee( "John", new DateTime(2008, 1, 1)),
                new Employee( "Mary", new DateTime(2007, 8, 5)),
                new Employee( "Paul", new DateTime(2003, 3, 14))
            };
            list.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(list_CollectionChanged);
            grid.DataSource = list;
        }

        void list_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                DispatcherOperationCallback dispatcher = delegate(object context)
                {
                    DataRecord dr = grid.GetRecordFromDataItem(context, false) as DataRecord;

                    if (dr != null)
                    {
                        dr.IsSelected = true;
                        grid.ExecuteCommand(DataPresenterCommands.DeleteSelectedDataRecords);
                    }

                    return null;
                };
                this.Dispatcher.BeginInvoke(dispatcher, DispatcherPriority.ApplicationIdle, list[list.IndexOf(e.NewItems[0] as Employee)]);
            }
        }

        private void grid_RecordsDeleting(object sender, Infragistics.Windows.DataPresenter.Events.RecordsDeletingEventArgs e)
        {
            // during the RecordsDeleting the records have not been deleted. in order to 
            // allow the end user to recreate those records, you must provide a custom
            // undeletestrategy that can store the data needed before the delete and 
            // perform the undeletion when requested (when the user attempts to perform
            // an undo). this strategy is a basic one that will reinsert the original
            // item into the containing list so this would work with ibindinglist<t> and 
            // observablecollection<t> type classes. this will not work with datatable/
            // dataset. for more examples please refer to the advanced undo sample in 
            // the feature browser sample
            e.UndeleteStrategy = new ListUndeleteStrategy(e.Records);
        }

        private void btnUndo_Click(object sender, RoutedEventArgs e)
        {
            grid.ExecuteCommand(DataPresenterCommands.Undo);
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            list.Add(new Employee("New", DateTime.Now));
        }
    }
}
